# stats.tcl
#	system statistics frame 


loadsource stats.common

set NUM 16

set ITEM(0) "Data Buffers"
set ITEM(1) "Total data words"
set ITEM(2) "Good Events"
set ITEM(3) "Histogram increments"
set ITEM(4) "Buffers transmitted"
set ITEM(5) "Total data transmitted"
set ITEM(6) "Null events"
set ITEM(7) "Event Number ReSync"
set ITEM(8) "Bad Time"
set ITEM(9) "Bad data"
set ITEM(10) "Event not acceptable"
set ITEM(11) "Item skipped"
set ITEM(12) "Bad D2VB block"
set ITEM(13) "Bad D2VB block length"
set ITEM(14) "Data transmit error"
set ITEM(15) "Waiting for network"

set BADCODE 24

set MoreText(1) "VRE has detected an error (q bit set)"
set MoreText(2) "Invalid crate number"
set MoreText(3) "Word count too short"
set MoreText(4) "Data word received is not the required token"

set MoreText(5) "VRE has detected an error (q bit set)"
set MoreText(6) "Incorrect BusID (group)"
set MoreText(7) "Subevent already received for this crate"
set MoreText(8) "Inconsistent 4 bit event check field"
set MoreText(9) "Word count too short"
set MoreText(10) "Data word received is not the required token"

set MoreText(11) "VRE has detected an error (q bit set)"
set MoreText(12) "Incorrect BusID (group)"
set MoreText(13) "Incorrect crate number"
set MoreText(14) "Inconsistent 4 bit event check field"
set MoreText(15) "Inconsistent 4 bit event check field"
set MoreText(16) "Incorrect event number"
set MoreText(17) "Data word received is not the required token"

set MoreText(18) "VRE has detected an error (q bit set)"
set MoreText(19) "Incorrect BusID (group)"
set MoreText(20) "Incorrect crate number"
set MoreText(21) "Incorrect event number"
set MoreText(22) "Data word received is not the required token"

set MoreText(23) "program logic error"
set MoreText(24) "Data for reserved group (0 or >= 248)"
set MoreText(25) "Too many Titris data words (>3)"

set MoreText(26) "VRE has detected an error (q bit set)"
set MoreText(27) "Incorrect BusID (group)"
set MoreText(28) "Incorrect crate number"
set MoreText(29) "Not all crates have given data"
set MoreText(30) "One or more invalid crates detected"
set MoreText(31) "Too few Titris data words (!=3)"
set MoreText(32) "Data word received is not the required token"

set MoreText(33) "program logic error"


proc custom_post_make {} {

   if {[first [inform sys BUstats]] != "invalid"} {
       :next-row T
       :vskip 5
       :message MRGE  :label "Merge Counters" :bold T
       :text MRGE1 :label "Event Merged Count" :w 10 :deaf T
       :text MRGE2 :label "Event Merged Error Count" :w 10 :deaf T
   }

   :next-row T
   :button MORE :label "More Statistics" :command click_more
   :message WARNING :label "" :bold T
}

proc custom_clear {} {:modify WARNING :label ""}

proc custom_redisplay {} {
    global Stats Rates
    
    if {$Rates(0) > 0 && $Rates(2) == 0} then {
       :modify WARNING :label "WARNING - You are receiving data but have NO good events. Try STOP then GO"
    } else {
       :modify WARNING :label ""
    }
    
    set stats [inform sys BUstats]
    if {[first $stats] != "invalid"} {
       :modify MRGE1 :value [first $stats]
       :modify MRGE2 :value [second $stats]
    }
}

set MoreVisible 0

proc click_more {} {
   global MoreVisible MoreText BADCODE

   set win [self frame-name].more

   if {$MoreVisible == 0} {

       :frame $win :label  "Data Formatting Statistics"  :transient T
       :layout T

       :galley G :rows 40 :cols 80 :deaf T :font fixed :strings ""
   } else {
       :modify G :empty T
   }

   :modify G :strings "Zero counters are suppressed\n"
   :modify G :strings "looking for start event token xx00 xxxx 1111 1xxx\n"

   for {set i 1} {$i <= 4} {incr i} {
      set stat [read-Stat [expr $i+$BADCODE]]
      if {$stat != 0} {:modify G :strings "[format "%3d %60s %8d" $i "$MoreText($i)" $stat]\n"}
   }

   :modify G :strings "looking for start subevent token xx00 xxxx 1111 1xxx\n"

   for {set i 5} {$i <= 10} {incr i} {
      set stat [read-Stat [expr $i+$BADCODE]]
      if {$stat != 0} {:modify G :strings "[format "%3d %60s %8d" $i "$MoreText($i)" $stat]\n"}
   }

   :modify G :strings "looking for VRE EvNum LSW token xx01 xxxx 1111 1xxx\n"

   for {set i 11} {$i <= 17} {incr i} {
      set stat [read-Stat [expr $i+$BADCODE]]
      if {$stat != 0} {:modify G :strings "[format "%3d %60s %8d" $i "$MoreText($i)" $stat]\n"}
   }

   :modify G :strings "looking for VRE EvNum MSW token xx10 xxxx 1111 1xxx\n"

   for {set i 18} {$i <= 22} {incr i} {
      set stat [read-Stat [expr $i+$BADCODE]]
      if {$stat != 0} {:modify G :strings "[format "%3d %60s %8d" $i "$MoreText($i)" $stat]\n"}
   }

   :modify G :strings "looking for data\n"

   for {set i 23} {$i <= 25} {incr i} {
      set stat [read-Stat [expr $i+$BADCODE]]
      if {$stat != 0} {:modify G :strings "[format "%3d %60s %8d" $i "$MoreText($i)" $stat]\n"}
   }

   :modify G :strings "looking for VRE end subevent token xx11 xxxx 1111 1xxx\n"

   for {set i 26} {$i <= 33} {incr i} {
      set stat [read-Stat [expr $i+$BADCODE]]
      if {$stat != 0} {:modify G :strings "[format "%3d %60s %8d" $i "$MoreText($i)" $stat]\n"}
   }

   :modify G :see 0

   if {$MoreVisible == 0} {
       :layout B; :set fill none; :set anchor center
       :button B  :label "Dismiss" :command click_nomore
       :show T
   }

   if {![:query_Mapped $win]} {open-frame $win}
   set MoreVisible 1

}

proc click_nomore {} {
    global MoreVisible 

    destroy [self frame-name].more
    set MoreVisible 0
}




