 # titris.tcl---RISING TITRIS Control module control frame



set frame "TITRIS"
set frametitle "TITRIS Control module"
set icontitle "TITRIS"

set modulename titris
set module 1

loadsource common.vme


set directory /nfs/groups/daq/usr/kurz/mbstest/rising/time_stamp_module/handling
set program ts_mod_RIO3
set HECTOR R3-9
set HectorInUse 1
set FRS R3-8
set FRSInUse 1


proc set-register {register v} {
    global modulename module
    write-vme-register $modulename#$module.$register $v
    click-redisplay
}

proc click-status {v} {set-register Status $v}
proc click-control {v} {set-register Control $v}
proc click-datalw {v} {set-register Data.LW $v}
proc click-datamw {v} {set-register Data.MW $v}
proc click-datahw {v} {set-register Data.HW $v}

proc click-master {args} {set-register Control 4}
proc click-slave  {args} {set-register Control 0x20} 
proc click-go     {args} {set-register Control 4; set-register Control 2}
proc click-halt   {args} {set-register Control 4; set-register Control 0x20}

proc click-calibrate {v} {
    global modulename module

    set success 0
    set-footer "Calibrating cable delay - please wait"

    write-vme-register $modulename#$module.Control 0x20

    write-vme-register $modulename#$module.Control 0x100

    for {set i 0} {$i < 5} {incr i} {
        after 1000
        set r [expr [read-vme-register $modulename#$module.Control] & 0xffff]
        if {$r == [expr 0x8900]} {
            set success 1 
            break
        }
    }

    :modify DELR :value [read-vme-register $modulename#$module.CableDelay]

    if {$success} {
       set-footer "OK: cable delay calibrated"
    } else {
       set-footer "ERROR: calibrating cable delay"
    }
    return $success
}


proc click-software {} {
   global modulename
   inform [set modulename]sftwre start
}

proc click-redisplay {} {
    global FRSInUse HectorInUse
 
    LOCAL_display

    if {$FRSInUse} {FRS_display}
    if {$HectorInUse} {HECTOR_display}
}

proc calibrate_all {} {

    set r [midas-confirmation "You are about to recalibrate the TITRIS system\n\
                          Are you sure you wish to continue"]
    if {$r == 0} then {return}


    log "[date]: Calibrating TITRIS system"
    titris_calibrate
    log "[date]: Finished"
}

proc save_preferences args {
    global preferences
    global directory program HECTOR HectorInUse FRS FRSInUse

    set preferences(directory) $directory
    set preferences(program) $program
    set preferences(HECTOR) $HECTOR
    set preferences(FRS) $FRS
    set preferences(HectorInuse) $HectorInUse
    set preferences(FRSInUse) $FRSInUse

    update_preferences
}

proc start args {
    global started CurrentVMEServer modulename module
    global preferences
    global directory program HECTOR HectorInUse FRS FRSInUse

   load_preferences

   if {[info exists preferences(directory)]} then {
       set directory $preferences(directory)
       set program $preferences(program)
       set HECTOR $preferences(HECTOR)
       set FRS $preferences(FRS)
       set HectorInUse $preferences(HectorInuse)
       set FRSInUse $preferences(FRSInUse)
   }

   if {$started == 0} then {
       make-frame
       EGbindings
   }
   set started 1
   open-frame

   clear-footer
   clear-busy
   set EGreport 0

   set CurrentVMEServer [VMEmodule2server $modulename#$module]
   click-redisplay
}

proc click-quit {} {close-frame}

proc finish args {destroy-frame}

proc click arg {
   global debug actionlist
   global directory program HECTOR HectorInUse FRS FRSInUse
   if {$debug} then {insert-debug "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]

   if {$c==":quit"} then {click-quit; return}
   switch -glob $w {
      ENGINEERING       {click-$v}
      DIRECTORY   {set directory $v}
      PROGRAM     {set program $v}
      HECTOR      {set HECTOR $v}
      FRS         {set FRS $v}
      HECTORINUSE {set HectorInUse $v}
      FRSINUSE    {set FRSInUse $v}
      default {
        set p "click-[lowercase $w]"
        if {[info procs $p] == "$p"} then {$p $v} else {set-footer "no handler for widget $w"}
      }
   }
}


proc make-frame {} {
  global env
  global directory program HECTOR HectorInUse FRS FRSInUse

set rows 8
set cols 80

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/vme_module" \
      :icon-name "[self icon-title]" 

   :panel MODULE  :layout T :padx 10 :pady 10
     :layout H; :row-gap 10; :col-gap 10

     :message M1 :label "Local" :bold T

     :text STATUS   :label "Status register"  :w 6 :deaf T
     :text CONTROL  :label "Control register" :w 6 :deaf T
     :text DELR     :label "Cable Delay"  :deaf T  :w 6
     :next-row T

     :message M2 :label "Data Register" :bold  T
     :text DATAHW  :label "High Word"    :w 6 :deaf T
     :text DATAMW  :label "Middle Word"  :w 6 :deaf T
     :text DATALW  :label "Low Word"     :w 6 :deaf T

     :next-row T
     :button MASTER    :label "Set module to master mode"
     :button SLAVE     :label "Set module to slave mode"
     :button CALIBRATE :label "Calibrate cable delay"

     :next-row T
     :button GO    :label "GO"  :w 8
     :button HALT  :label "HALT"  :w 8
     :checkbox CGO  :label "GO set" :deaf T
     :checkbox CMTM :label "Master set" :deaf T
     :checkbox CRES :label "Reset set" :deaf T

     :next-row T
     :text DIRECTORY :label "directory" :w 70 :value $directory :valuex 10c
     :next-row T
     :text PROGRAM   :label "program"   :w 20 :value $program :valuex 10c

     :next-row T
     :message H1  :label "Hector" :bold T
     :text HECTOR :label "Host" :w 12 :value $HECTOR
     :checkbox HECTORINUSE :label "In Use" :value $HectorInUse
     :text HECTORSTATUS  :label "Status register" :w 6 :deaf T
     :text HECTORCONTROL :label "Control register" :w 6 :deaf T
     :next-row T
     :message H2 :label "Data Register" :bold T
     :text HECTORDATAHW  :label "High Word" :w 6 :deaf T
     :text HECTORDATAMW  :label "Middle Word" :w 6 :deaf T
     :text HECTORDATALW  :label "Low Word" :w 6 :deaf T

     :next-row T
     :message F1  :label "FRS" :bold T
     :text FRS :label "Host" :w 12 :value $FRS
     :checkbox FRSINUSE :label "In Use" :value $FRSInUse
     :text FRSSTATUS  :label "Status register" :w 6 :deaf T
     :text FRSCONTROL :label "Control register" :w 6 :deaf T
     :next-row T
     :message F2 :label "Data Register" :bold T
     :text FRSDATAHW  :label "High Word" :w 6 :deaf T
     :text FRSDATAMW  :label "Middle Word" :w 6 :deaf T
     :text FRSDATALW  :label "Low Word" :w 6 :deaf T

  :panel P2 :layout V
    :galley LIST :rows $rows :cols $cols :deaf T  :font variable

   :panel CLICK :layout B :padx 10 :after FOOTER
     :layout H; :col-gap 30; :row-gap 10
      :vskip 10
      :button REDISPLAY  :command click-redisplay
      :menubutton ENGINEERING :label "Engineering"  :menu [list software]
      :button DOALL :label "Calibrate TITRIS system" :command calibrate_all
      :button SAVE :label "Save Options" :command save_preferences

  open-frame

  :galley_size LIST $rows $cols

}

proc click-configure {W} {:galley_resize LIST}

proc log {s} {:modify LIST :strings $s\n}

proc titris_calibrate {} {
    global directory program HECTOR HectorInUse FRS FRSInUse

#    first set all TITRIS modules to slave

    log "[date]: Setting local TITRIS to slave mode"
    set result [click-slave]
    log "[date]: $result"

    if {$FRSInUse} {
       log "[date]: Setting FRS TITRIS to slave mode"
       set z [catch {exec rsh $FRS $directory/$program -s} result]
       log "[date]: $result"
    }

    if {$HectorInUse} {
       log "[date]: Setting HECTOR TITRIS to slave mode"
       set z [catch {exec rsh $HECTOR $directory/$program -s} result]
       log "[date]: $result"
    }

#    first master TITRIS module to master (assumed to be us)

    log "[date]: Setting local TITRIS to master mode"
    set result [click-master]
    log "[date]: $result"

#    now calibrate all slaves 

    if {$FRSInUse} {
       log "[date]: Calibrating FRS TITRIS"
       set z [catch {exec rsh $FRS $directory/$program -c} result]
       log "[date]: $result"
       set r [second [split $result \n]]
       if {[string range $r 0 2] != "OK:"} then {
           midas-warning "$result\nResponse received from FRS ($FRS) is not as expected"
       }
    }

    if {$HectorInUse} {
       log "[date]: Calibrating HECTOR TITRIS"
       set z [catch {exec rsh $HECTOR $directory/$program -c} result]
       log "[date]: $result"
       set r [second [split $result \n]]
       if {[string range $r 0 2] != "OK:"} then {
           midas-warning "$result\nResponse received from HECTOR ($HECTOR) is not as expected"
       }
    }

#    set master TITRIS module to GO  (assumed to be us)

    log "[date]: Setting local TITRIS to GO"
    set result [click-go]
    log "[date]: $result"

}

proc LOCAL_display {} {
    global module modulename

    set register Status
    set vl [read-vme-register $modulename#$module.$register]
    :modify STATUS :value  [format "0x%04x" [expr $vl & 0xffff]]

    set register Control
    set vl [read-vme-register $modulename#$module.$register]
    :modify CONTROL :value  [format "0x%04x" [expr $vl & 0xffff]]

    set register Control.GO
    set vl [read-vme-register $modulename#$module.$register]
    :modify CGO :value $vl

    set register Control.MASTER
    set vl [read-vme-register $modulename#$module.$register]
    :modify CMTM :value $vl

    set register Control.RESET
    set vl [read-vme-register $modulename#$module.$register]
    :modify CRES :value $vl

    set register CableDelay
    set vl [read-vme-register $modulename#$module.$register]
    :modify DELR :value $vl

    set register Data.LW
    set vl [read-vme-register $modulename#$module.$register]
    :modify DATALW :value  [format "0x%04x" [expr $vl & 0xffff]]

    set register Data.MW
    set vl [read-vme-register $modulename#$module.$register]
    :modify DATAMW :value  [format "0x%04x" [expr $vl & 0xffff]]

    set register Data.HW
    set vl [read-vme-register $modulename#$module.$register]
    :modify DATAHW :value  [format "0x%04x" [expr $vl & 0xffff]]
}

proc FRS_display {} {
    global FRS directory program

       log "[date]: Getting FRS TITRIS time stamp"
       set z [catch {exec rsh $FRS $directory/$program -t} result]
       log "[date]: $result"
       if {[string range $result 0 11] != "TIME STAMP :"} then {
           midas-warning "$result\nResponse received from FRS ($FRS) is not as expected"
       } else {
          :modify FRSDATAHW :value  "0x[string range $result 15 18]"
          :modify FRSDATAMW :value  "0x[string range $result 20 23]"
          :modify FRSDATALW :value  "0x[string range $result 25 28]"
       }

       log "[date]: Getting FRS TITRIS status"
       set z [catch {exec rsh $FRS $directory/$program -r} result]
       log "[date]: $result"
       set r [first [split $result \n]]
       if {[string range $r 0 6] != "STATUS:"} then {
           midas-warning "$result\nResponse received from FRS ($FRS) is not as expected"
       } else {
          :modify FRSSTATUS :value  "[string range $r 13 end]"
       }
       set r [second [split $result \n]]
       if {[string range $r 0 7] != "CONTROL:"} then {
           midas-warning "$result\nResponse received from FRS ($FRS) is not as expected"
       } else {
          :modify FRSCONTROL :value  "[string range $r 13 end]"
       }

}


proc HECTOR_display {} {
    global HECTOR directory program

       log "[date]: Getting HECTOR TITRIS time stamp"
       set z [catch {exec rsh $HECTOR $directory/$program -t} result]
       log "[date]: $result"
       if {[string range $result 0 11] != "TIME STAMP :"} then {
           midas-warning "$result\nResponse received from HECTOR ($HECTOR) is not as expected"
       } else {
          :modify HECTORDATAHW :value  "0x[string range $result 15 18]"
          :modify HECTORDATAMW :value  "0x[string range $result 20 23]"
          :modify HECTORDATALW :value  "0x[string range $result 25 28]"
       }

       log "[date]: Getting HECTOR TITRIS status"
       set z [catch {exec rsh $HECTOR $directory/$program -r} result]
       log "[date]: $result"
       set r [first [split $result \n]]
       if {[string range $r 0 6] != "STATUS:"} then {
           midas-warning "$result\nResponse received from HECTOR ($HECTOR) is not as expected"
       } else {
          :modify HECTORSTATUS :value  "[string range $r 13 end]"
       }
       set r [second [split $result \n]]
       if {[string range $r 0 7] != "CONTROL:"} then {
           midas-warning "$result\nResponse received from HECTOR ($HECTOR) is not as expected"
       } else {
          :modify HECTORCONTROL :value  "[string range $r 13 end]"
       }

}


